
--- Tracks spending on repairs.
local _state = {
	cost = 0, -- total cost of repairs
	guildBudget = 0, -- guild money available for repairs
	guildSpent = 0, -- guild money spent on repairs
	playerBudget = 0, -- player money available for repairs
	playerSpent = 0, -- player money spent on repairs
}

--- Used for getting repair costs of items.
local _tooltip = CreateFrame("GameTooltip", "hcRepairBotTooltip", UIParent, "GameTooltipTemplate")

--- Computes the repair cost of an item in the player's bags.
local function _GetContainerItemRepairCost(container, slot)
	if not InRepairMode() then
		ShowRepairCursor()
	end
	
	local hasCooldown, repairCost = addon.tooltip:SetBagItem(container, slot)
	
	return repairCost or 0
end

--- Gets the amount of guild money available for repairs.
-- If the player is not in a guild, or does not have access to guild 
-- repair funds, or has exceeded the daily guild repair limit, or 
-- does not pass the other criteria set in this addon's configuration 
-- options, this will return zero. When money is available, the 
-- amount is returned.
-- @return the guild repair budget, in copper.
local function _GetGuildRepairBudget()
	if not IsInGuild() then
		return 0
	end
	
	if not CanGuildBankRepair() then
		return 0
	end
	
	inGroup, _, _, _ = InGuildParty()
	if not inGroup then
		return 0
	end
	
	return GetGuildBankWithdrawMoney()
end

-- Calculates the repair cost of an item in the player's inventory.
local function _GetInventoryItemRepairCost(slot)
	if not InRepairMode() then
		ShowRepairCursor()
	end
	
	local hasItem, _, repairCost = addon.tooltip:SetInventoryItem("player", slot)
	if hasItem then
		return repairCost
	end
	
	return 0
end

--- Repairs all items using guild or player funds.
-- This should only be called when the repair budget allows for a full 
-- repair. 
-- @param guild true if guild money should be used, false for player 
-- money.
local function _RepairAllItems(guild)
	if guild then
		assert(_state.guildBudget >= _state.cost)
		RepairAllItems(1)
		_state.guildSpent = _state.guildSpent + _state.cost
		_state.guildBudget = _state.guildBudget - _state.cost
	else
		assert(_state.playerBudget >= _state.cost)
		RepairAllItems()
		_state.playerSpent = _state.playerSpent + _state.cost
		_state.playerBudget = _state.playerBudget - _state.cost
	end
	
	_state.cost = 0
end

--- Repairs a single item in the player's inventory.
-- Single-item repairs are only possible using player funds. 
-- @param slot an inventory slot.
local function _RepairInventoryItem(slot)
	local link = GetInventoryItemLink("player", slot)
	if not link then
		return
	end
	
	local cost = _GetInventoryItemRepairCost(slot)
	if (cost > _state.playerBudget) or (cost <= 0) then
		return
	end
	
	if not InRepairMode() then
		ShowRepairCursor()
	end
	PickupInventoryItem(slot)
	
	_state.playerSpent = _state.playerSpent + cost
	_state.playerBudget = _state.playerBudget - cost
	_state.cost = _state.cost - cost
end

--- Repairs individual items in the player's inventory.
-- This will repair items until the total remaining cost of repairs 
-- is less than the guild repair budget, allowing guild repair funds 
-- to be used as much as possible. This walks through inventory 
-- slots in the internal game order. Called without any arguments, 
-- this will first repair they player's weapons, the the rest of 
-- the player's gear.
-- @param first the first inventory slot to repair.
-- @param last the last inventory slot to repair.
local function _RepairSomeItems(first, last)

	if not first then
		_RepairSomeItems(INVSLOT_MAINHAND, INVSLOT_OFFHAND)
		_RepairSomeItems(INVSLOT_FIRST_EQUIPPED, INVSLOT_LAST_EQUIPPED)
		return
	end
	
	if not last then
		last = first
	end
	
	for slot = first, last do
		_RepairInventoryItem(slot)
		if _state.cost <= _state.guildBudget then
			break
		end
	end
end


-- Construct the addon.
local addon = CreateFrame("Frame", "hcRepairBotFrame")

--- Dispatches events to this addon by calling the method with the 
-- same name as the given event.
function addon:OnEvent(event, ...)
	if self[event] then
		self[event](self, event, ...)
	end
end

-- Register the event handler and initial events.
addon:SetScript("OnEvent", addon.OnEvent)






--- Repairs items using guild and player money.
-- If the guild's repair budget is insufficient to cover the full cost 
-- of repairs, player money will be used to repair some items until 
-- the guild budget can be used to cover the full cost of repairs. If 
-- the player budget is inadequate to do this, player money will 
-- continue to be spent until all items are repaired or the player 
-- budget is exhausted.
-- @param cost the total cost of repairs,
-- @param guildBudget the guild money available for repairs.
-- @param playerBudget the player money available for repairs.
-- @return the remaining repair cost.
function _RepairUsingAllMoney()
	if _state.cost > _state.guildBudget then
		_RepairSomeItems()
	end
	_RepairUsingGuildMoney()
	_RepairUsingPlayerMoney()
end

--- Repairs items using guild money.
-- If the guild's repair budget is insufficient to cover the full cost 
-- of repairs, no action will be taken.
-- @param cost the total cost of repairs.
-- @param budget the repair budget.
-- @return the remaining cost of repairs.
function _RepairUsingGuildMoney()
	if _state.cost <= _state.guildBudget then
		_RepairAllItems(true)
	end
end

--- Repairs items using player money. 
-- If the player's repair budget is insufficient to cover the full cost 
-- of repairs, this will instead repair only what items can fit within 
-- that budget.
-- @param cost the total cost of repairs.
-- @param budget the repair budget.
-- @return the remaining cost of repairs.
function _RepairUsingPlayerMoney()
	if _state.cost <= _state.playerBudget then
		_RepairAllItems(false)
	else
		_RepairSomeItems()
	end
end

--- Clears the internal state table.
-- Sets the repair cost, budgets, and expenditures to zero.
function _ClearState()
	_state.cost = 0
	_state.guildBudget = 0
	_state.guildSpent = 0
	_state.playerBudget = 0
	_state.playerSpent = 0
end

--- Updates the internal state table.
-- This resets the repair cost, budgets, and expenditures when 
-- interacting with a new merchant.
function _UpdateState()
	_state.cost = GetRepairAllCost()
	_state.guildBudget = _GetGuildRepairBudget()
	_state.guildSpent = 0
	_state.playerBudget = GetMoney()
	_state.playerSpent = 0
end

--- Fires when the player ends interaction with a merchant.
-- This clears the internal state table.
function addon:MERCHANT_CLOSED()
	_ClearState()
end

--- Fires when the player begins interaction with a merchant.
-- This will determine whether or not the merchant is capable of 
-- performing repairs. If so, guild and/or player money will be used 
-- to perform such repairs, subject to the addon configuratoin 
-- options.
function addon:MERCHANT_SHOW()
	if not CanMerchantRepair() then
		return
	end

	_UpdateState()
	
	if _state.cost <= 0 then
		return
	end

	if _state.guildBudget > 0 then
		if _state.playerBudget > 0 then
			_RepairUsingAllMoney()
		else
			_RepairUsingGuildMoney()
		end
	else
		if _state.playerBudget > 0 then
			_RepairUsingPlayerMoney()
		end
	end

	if _state.guildSpent > 0 then
		local spent = GetCoinTextureString(_state.guildSpent)		
		ChatFrame_DisplaySystemMessageInPrimary(("Your guild spent %s on repairs."):format(spent))
	end

	if _state.playerSpent > 0 then
		local spent = GetCoinTextureString(_state.playerSpent)
		ChatFrame_DisplaySystemMessageInPrimary(("You spent %s on repairs."):format(spent))
	end
	
	ResetCursor()
end

--- Registers events once the addon is loaded.
-- The ADDON_LOADED event is unregistered to prevent redundant calls to this
-- handler.
function addon:ADDON_LOADED(event, name)
	if name ~= "hcRepairBot" then
		return
	end

	self:UnregisterAllEvents()

	self:RegisterEvent("MERCHANT_CLOSED")
	self:RegisterEvent("MERCHANT_SHOW")
end

addon:RegisterEvent("ADDON_LOADED")

